/*
 * Created on Feb 11, 2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package analysis.gps.server.routeFinder;
import javax.ejb.*;
import javax.naming.*;
import java.rmi.*;
import drasys.or.graph.*;
import drasys.or.graph.sp.*;
import java.util.*;
import org.apache.log4j.*;

/**
 * @ejb.bean name="RouteFinder"
 *	jndi-name="RouteFinderBean"
 *	type="Stateless" 
 * 
 *--
 * This is needed for JOnAS.
 * If you are not using JOnAS you can safely remove the tags below.
 * @jonas.bean ejb-name="RouteFinder"
 *	jndi-name="RouteFinderBean"
 * 
 *--
 **/
public abstract class RouteFinderBean implements SessionBean {
	static Logger logger=Logger.getLogger(RouteFinderBean.class); 
	private SparseGraph city;	
	/**
	 * @ejb.interface-method
	 *	view-type="remote" 
	 *The findRoute method finds the shortest path between the intial position and the destination. It takes all of the vertex
	 * in that path and returns them in a Vector.
	 **/	
	public Vector findRoute(VertexI initialPosition,VertexI destination) throws RemoteException{
		CityFactoryHome cityFactoryHome;
		CityFactory cityFactory;
		SparseGraph c;
		Dijkstra algorithm; //This is the class that calculatest the shortest path
		Vector path=new Vector();
		VertexI nearest;
		Enumeration enum;
		VertexI vertex;
		String pathStr="**"+initialPosition.getValue().toString()+"\n";
		if(initialPosition.equals(destination))throw new RemoteException("The initial position and destination are the same");
		try{
			cityFactoryHome= (CityFactoryHome)getContext().lookup(CityFactoryHome.JNDI_NAME);
			cityFactory=cityFactoryHome.create();
			c=cityFactory.getCity();
			algorithm = new Dijkstra(c);
			algorithm.setCandidate(false); //With this, we initialize the algorithm, there are no final vertices now
			path=new Vector();
			algorithm.setCandidate(destination.getKey(),true); //Now we set the destination as the final vertex
			algorithm.generatePathsFrom(initialPosition.getKey()); 
			nearest = algorithm.getNearestCandidate();			
			path = algorithm.getPath(nearest);
			enum = path.elements();
			vertex = (VertexI)enum.nextElement();

			while(enum.hasMoreElements()){
				enum.nextElement();// Remove Edge //
				vertex = (VertexI)enum.nextElement();
				pathStr += " - " + vertex.getValue()+"\n";
			}			
			pathStr+="**"+destination.getValue().toString()+"\n";
			logger.info("Shortes path:\n"+pathStr);
		}catch(VertexNotFoundException e){
			System.out.println("Vertex not found");
			//e.printStackTrace();
		}catch(InvalidPropertyException e){
			System.out.println("Invalid property");
		}catch(CreateException e){
			throw new RemoteException("The satellite was unable to obtain the path information.");
		}catch(NamingException e){
			throw new RemoteException("Satellite malfunction due to wrong binding. Please contact customer service.");
		}
		return path;
	}
	
	/**
	 * @ejb.interface-method
	 *	view-type="remote" 
	 **/
	public VertexI getInitialPosition() throws RemoteException{
		Random randomizer;
		int randomMapPosition;
		Enumeration vertices;
		VertexI initialVertex;
		int verticesSize;
		
		try{
			if(city==null){
				logger.info("Map information unavailable to the satellite. Obtaining city map");
				getCity();
			}
			logger.info("Assigning position to the client");
			randomizer=new Random();
			verticesSize=city.sizeOfVertices();
			if(verticesSize<=0) throw new RemoteException("City vertices not available for obtaining an initail position");
			randomMapPosition=randomizer.nextInt(verticesSize);
			vertices=city.vertices();
			for(int i=0;i<randomMapPosition;i++){
				vertices.nextElement();
			}
			initialVertex=(VertexI)vertices.nextElement();
			logger.info("Assigned position:"+initialVertex.getValue()+initialVertex.getKey());
			return initialVertex;
		}catch(CreateException e){
			throw new RemoteException("The satellite was unable to obtain the map information.");
		}catch(NamingException e){
			throw new RemoteException("Satellite malfunction due to wrong binding. Please contact customer service.");
		}		
	}

	/**
	 * @ejb.interface-method
	 *	view-type="remote" 
	 **/	
	public VertexI getPosition(VertexI currentPosition, VertexI suggestedNextPosition) throws RemoteException{
		try{
			if(city==null){
				logger.info("Map information unavailable to the satellite. Obtaining city map");
				getCity();
			}		
			//There is an 80% probability that the vehicle will follow the suggested path
			Enumeration outEdges=currentPosition.outEdges();
			while(outEdges.hasMoreElements()){
				EdgeI e=(EdgeI)outEdges.nextElement();
				if(e.getToVertex().equals(suggestedNextPosition)){
					if(((new Random()).nextInt(100))>80){
						return e.getToVertex();
					}
				}
			}
		}catch(CreateException e){
			throw new RemoteException("The satellite was unable to obtain the map information.");
		}catch(NamingException e){
			throw new RemoteException("Satellite malfunction due to wrong binding. Please contact customer service.");
		}				
		return suggestedNextPosition;
	}

	private void getCity() throws RemoteException,CreateException,NamingException{
		CityFactoryHome cityFactoryHome = (CityFactoryHome)getContext().lookup(CityFactoryHome.JNDI_NAME);
		CityFactory cityFactory=cityFactoryHome.create();
		this.city=cityFactory.getCity();
	}
	
	
	private InitialContext getContext() throws NamingException {
		Hashtable props = new Hashtable();

		props.put(
				InitialContext.INITIAL_CONTEXT_FACTORY,
		"org.jnp.interfaces.NamingContextFactory");
		props.put(InitialContext.PROVIDER_URL, "jnp://localhost:1099");

		// This establishes the security for authorization/authentication
		// props.put(InitialContext.SECURITY_PRINCIPAL,"username");
		// props.put(InitialContext.SECURITY_CREDENTIALS,"password");

		InitialContext initialContext = new InitialContext(props);
		return initialContext;
	}
	/**
	 * @ejb.interface-method
	 *	view-type="remote" 
	 **/
	public VertexI getRandomDestination() throws RemoteException{ 
		return this.getInitialPosition();
	}
/**
 * @ejb.interface-method
 *	view-type="remote" 
**/
public VertexI getFinalPosition(Integer posId) throws RemoteException{ 
	Random randomizer;
	int randomMapPosition;
	Enumeration vertices;
	VertexI initialVertex=null;
	int verticesSize;
	
	try{
		if(city==null){
			logger.info("Map information unavailable to the satellite. Obtaining city map");
			getCity();
		}
		logger.info("get position position to the client");
		verticesSize=city.sizeOfVertices();
		if(verticesSize<=0) throw new RemoteException("City vertices not available for obtaining an initail position");
		vertices=city.vertices();
		for(int i=0;i< verticesSize;i++){
			initialVertex=(VertexI)vertices.nextElement();
			if(initialVertex.getIndex()==posId.intValue()) break;
		}
		
		logger.info("Assigned position:"+initialVertex.getValue()+initialVertex.getKey());
		return initialVertex;
	}catch(CreateException e){
		throw new RemoteException("The satellite was unable to obtain the map information.");
	}catch(NamingException e){
		throw new RemoteException("Satellite malfunction due to wrong binding. Please contact customer service.");
	}		
}
}